package com.chinaitop.depot.business.controller;


import java.io.IOException;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

import javax.servlet.http.HttpServletRequest;

import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.http.MediaType;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RequestMethod;
import org.springframework.web.bind.annotation.RestController;
import org.springframework.web.multipart.MultipartFile;

import com.chinaitop.depot.business.model.BusinessFile;
import com.chinaitop.depot.business.service.BusinessFileService;
import com.fasterxml.jackson.core.JsonParseException;
import com.fasterxml.jackson.databind.JsonMappingException;

import io.swagger.annotations.Api;
import io.swagger.annotations.ApiImplicitParam;
import io.swagger.annotations.ApiImplicitParams;
import io.swagger.annotations.ApiOperation;

/**
 * 粮库业务管理-文件
* <p>Title: BusinessPlanController</p>
* <p>Description: </p>
* @author yaoyabin
* @date 2017年12月8日 下午5:43:19
 */
@RestController
@RequestMapping(value="/depot/business/file")
@Api(value= "BusinessFileController", description = "文件")

public class BusinessFileController {
    @Autowired
    private BusinessFileService businessFileService;
    
    /**
     * 删除.
     * @param id
     * @return
     * @throws JsonParseException
     * @throws JsonMappingException
     * @throws IOException
     */
    @RequestMapping(value="/remove", produces = MediaType.APPLICATION_JSON_VALUE, method=RequestMethod.POST)
	@ApiOperation(value="删除", notes = "根据id删除")
	@ApiImplicitParams({
		@ApiImplicitParam(name = "id", value = "id", paramType = "form")
	})
    public Map<String, Object> remove(Integer id, HttpServletRequest request) throws JsonParseException, JsonMappingException, IOException {
        Map<String, Object> modelMap = new HashMap<String, Object>();
        // 服务器路径
        String rootPath = request.getSession().getServletContext().getRealPath("");
        if (id != null) {
            businessFileService.remove(id, rootPath);
        }
        modelMap.put("status", "success");
        return modelMap;
    }
    
    
    @RequestMapping(value="/uploadFile")
    public BusinessFile uploadFile(MultipartFile file, String type, Integer bid, Integer userId,Integer orgId,HttpServletRequest request) throws IllegalStateException, IOException {
        BusinessFile businessFile = businessFileService.uploadFile(file,type,bid, userId, orgId);
        return businessFile;
    }

    @RequestMapping(value="/updateFile", produces = MediaType.APPLICATION_JSON_VALUE, method=RequestMethod.POST)
    @ApiOperation(value="修改数据", notes = "修改数据保存的文件数据")
    @ApiImplicitParams({
            @ApiImplicitParam(name = "fileIds", value = "文件ID集合", paramType = "form"),
            @ApiImplicitParam(name = "bid", value = "主键id", paramType = "form")
    })
    public void updateFile(String fileIds, Integer bid, String fileType){
        businessFileService.updateFileByFileIdsAndBid(fileIds,bid,fileType);
    }

    @RequestMapping(value="/getList", produces = MediaType.APPLICATION_JSON_VALUE, method=RequestMethod.GET)
    @ApiOperation(value="按条件查询附件数据", notes = "按条件查询附件数据")
    @ApiImplicitParams({
    	@ApiImplicitParam(name = "bid", value = "主表id", paramType = "query"),
        @ApiImplicitParam(name = "fileType", value = "文件类型", paramType = "query"),
        @ApiImplicitParam(name = "order", value = "排序方式", paramType = "query"),
        @ApiImplicitParam(name = "orgId", value = "单位id", paramType = "query")
    })
    public List<BusinessFile> getList(Integer bid, String fileType, String order, Integer orgId){
    	List<BusinessFile> list = null;
  
    	list = businessFileService.getByBid(bid, fileType, orgId, order);

        return list;
    }
}
