package com.chinaitop.depot.business.service.impl;

import java.math.BigDecimal;
import java.text.ParseException;
import java.text.SimpleDateFormat;
import java.util.ArrayList;
import java.util.Date;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

import javax.annotation.Resource;

import org.apache.commons.lang.StringUtils;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;

import com.alibaba.fastjson.JSONObject;
import com.chinaitop.depot.act.service.ActTaskService;
import com.chinaitop.depot.agile.model.BusinessScedule;
import com.chinaitop.depot.agile.service.BusinessSceduleService;
import com.chinaitop.depot.business.mapper.BusinessDeliveryStorageNoticeMapper;
import com.chinaitop.depot.business.model.BusinessApproval;
import com.chinaitop.depot.business.model.BusinessContract;
import com.chinaitop.depot.business.model.BusinessContractExample;
import com.chinaitop.depot.business.model.BusinessDeliveryStorageNotice;
import com.chinaitop.depot.business.model.BusinessDeliveryStorageNoticeExample;
import com.chinaitop.depot.business.model.BusinessLading;
import com.chinaitop.depot.business.model.BusinessLadingExample;
import com.chinaitop.depot.business.model.BusinessDeliveryStorageNoticeExample.Criteria;
import com.chinaitop.depot.business.model.BusinessPlan;
import com.chinaitop.depot.business.model.BusinessStoreWareDetail;
import com.chinaitop.depot.business.model.UserInfo;
import com.chinaitop.depot.business.model.enums.AuditType;
import com.chinaitop.depot.business.model.enums.CrkNoticeStatus;
import com.chinaitop.depot.business.model.enums.StoreWareType;
import com.chinaitop.depot.business.model.enums.TaskType;
import com.chinaitop.depot.business.model.vo.BusinessDeliveryStorageNoticeAuditVO;
import com.chinaitop.depot.business.service.BusinessApprovalService;
import com.chinaitop.depot.business.service.BusinessContractService;
import com.chinaitop.depot.business.service.BusinessDeliveryStorageNoticeService;
import com.chinaitop.depot.business.service.BusinessLadingService;
import com.chinaitop.depot.business.service.BusinessPlanService;
import com.chinaitop.depot.business.service.BusinessStoreWareDetailService;
import com.chinaitop.depot.business.service.UserInfoService;
import com.github.pagehelper.PageHelper;
import com.github.pagehelper.PageInfo;
import com.google.common.collect.Maps;


@Service
public class BusinessDeliveryStorageNoticeServiceImpl implements BusinessDeliveryStorageNoticeService {
    @Resource
    private BusinessDeliveryStorageNoticeMapper businessDeliveryStorageNoticeMapper;
    @Resource
    private BusinessStoreWareDetailService businessStoreWareDetailService;
    //@Resource
    //private BasicCodeRuleService basicCodeRuleService;
    @Resource
    private BusinessLadingService ladingService;
    @Resource
    private BusinessPlanService planService;
    @Resource
    private BusinessContractService contractService;
    @Resource
    private BusinessSceduleService businessSceduleService;
    @Resource
    private ActTaskService actTaskService;
    @Resource
    private BusinessApprovalService businessApprovalService;
    @Resource
    private UserInfoService userInfoService;
    
    // 入库通知单枚举id,出入库系统使用.
    private static Integer STORAGENOTICEENUMID = 2816;
    // 出库通知单枚举id,出入库系统使用.
    private static Integer DELIVERYNOTICEENUMID = 2818;

    public List<BusinessDeliveryStorageNotice> queryByExample(BusinessDeliveryStorageNoticeExample example) {
        return businessDeliveryStorageNoticeMapper.selectByExample(example);
    }

    
    public BusinessDeliveryStorageNotice findById(Integer id) {
        return businessDeliveryStorageNoticeMapper.selectByPrimaryKey(id);
    }

    /**
     * 新增，如果是出库，需要修改子表.
     */
    @Transactional(readOnly = false)
    public Map<String, Object> add(BusinessDeliveryStorageNotice businessDeliveryStorageNotice, 
            List<BusinessStoreWareDetail> storeWareDetailList, Integer userId, String name ,Integer orgId,Integer depotInfoOrgId) {
        Map<String, Object> returnMap = new HashMap<String, Object>();
        /*//当前登录人的id
        Integer userId = null;
        //当前登录人的姓名
        String name = null;
        try {
            userId = userInfo.getUserId();
            name = userInfo.getRealName();
        } catch (Exception e) {
            e.printStackTrace();
            returnMap.put("error", "获取不到登录人信息，请重新登录！");
            return returnMap;
        }*/
        // 设置审批状态为，审批待审批.
        businessDeliveryStorageNotice.setAuditState(AuditType.UNSUBMIT.getAuditCode());
        // 设置登录人信息.
        businessDeliveryStorageNotice.setBillDate(new Date());
        businessDeliveryStorageNotice.setCreateTime(new Date());
        businessDeliveryStorageNotice.setBillMan(name);
        businessDeliveryStorageNotice.setCreater(name);
        businessDeliveryStorageNotice.setCreaterId(userId);
        businessDeliveryStorageNotice.setOrgId(orgId);
        // businessDeliveryStorageNotice.setBillUnit("经营一部");
        
        // 启用状态设定为执行中，启用
        businessDeliveryStorageNotice.setInApplication(CrkNoticeStatus.ZHIXINGZHONG.getCrkNoticeCode());
        
        // 流程任务类型.
        TaskType taskType = null;
        try {
            if ("1".equals(businessDeliveryStorageNotice.getBillType())) {
                // 入库
                taskType = TaskType.storageNotice;
                // 入库通知单枚举id,出入库系统使用.
                businessDeliveryStorageNotice.setBillTypeId(STORAGENOTICEENUMID);
                // 设置通知单编号.
                //businessDeliveryStorageNotice.setBillNumber(basicCodeRuleService.getCode(CodeRuleType.storageNotice, depotInfo.getOrgId()));
            } else if ("3".equals(businessDeliveryStorageNotice.getBillType())) {
                // 出库
                taskType = TaskType.deliveryNotice;
                // 出库通知单枚举id,出入库系统使用.
                businessDeliveryStorageNotice.setBillTypeId(DELIVERYNOTICEENUMID);
                // 设置通知单编号.
                //businessDeliveryStorageNotice.setBillNumber(basicCodeRuleService.getCode(CodeRuleType.deliveryNotice, depotInfo.getOrgId()));
            } else {
                throw new RuntimeException("通知单类型有误，保存失败！");
            }
        } catch (Exception e) {
            e.printStackTrace();
            returnMap.put("error", e.getMessage());
            return returnMap;
        }
        
        // 保存.
        businessDeliveryStorageNoticeMapper.insert(businessDeliveryStorageNotice);

        returnMap.put("id",businessDeliveryStorageNotice.getId());
        
        // 启动流程
        String startProcess1 = actTaskService.startProcessNew(taskType.getTaskType(),
                businessDeliveryStorageNotice.getId().toString(), name, userId.toString(), depotInfoOrgId.toString());
        JSONObject jsStr = JSONObject.parseObject(startProcess1);
        String processInstanceId = jsStr.getString("processInstanceId");
        String processDefinitionId = jsStr.getString("processDefinitionId");
        // 设置流程实例id和流程定义id到合同中.
        // 设置流程实例id.
        businessDeliveryStorageNotice.setProcessInstanceId(processInstanceId);
        // 设置流程定义id.
        businessDeliveryStorageNotice.setProcessDefinitionId(processDefinitionId);
        // 修改.
        businessDeliveryStorageNoticeMapper.updateByPrimaryKey(businessDeliveryStorageNotice);
        
        // 新增仓房货位数量质量标准子表.
        if (storeWareDetailList != null && storeWareDetailList.size() > 0) {
            for (BusinessStoreWareDetail storeWareDetail : storeWareDetailList) {
                storeWareDetail.setZid(businessDeliveryStorageNotice.getId());
                storeWareDetail.setType(StoreWareType.notice.getValue());
                storeWareDetail.setCreateTime(new Date());
                if("1".equals(businessDeliveryStorageNotice.getBillType())){ //入库
                	storeWareDetail.setOutCount("");
                	storeWareDetail.setOutPrice("");
                	storeWareDetail.setOutRemainingNumber("");
                }
                if("3".equals(businessDeliveryStorageNotice.getBillType())){ //出库
                	storeWareDetail.setInCount("");
                	storeWareDetail.setInPrice("");
                	storeWareDetail.setRemainingNumber("");
                }
                businessStoreWareDetailService.add(storeWareDetail);
            }
        }
        
        return returnMap;
    }

    /**
     * 修改，如果是出库，需要修改子表.
     */
    @Transactional(readOnly = false)
    public void update(BusinessDeliveryStorageNotice businessDeliveryStorageNotice,
            List<BusinessStoreWareDetail> storeWareDetailList) {
        businessDeliveryStorageNoticeMapper.updateByPrimaryKey(businessDeliveryStorageNotice);
        
        if (storeWareDetailList != null && storeWareDetailList.size() > 0) {
            for (BusinessStoreWareDetail storeWareDetail : storeWareDetailList) {
                storeWareDetail.setZid(businessDeliveryStorageNotice.getId());
                if (storeWareDetail.getId() == null) {
                    // 新增.
                    businessStoreWareDetailService.add(storeWareDetail);
                } else {
                    // 修改.
                    businessStoreWareDetailService.update(storeWareDetail);
                }
            }
        }
    }

    /**
     * 删除，主表和子表数据(出库).
     */
    public void remove(Integer id, String processInstanceId, String deleteReason,Integer planBid, Integer contractBid) {
//        if(planBid!=null && contractBid==null){
//        	//只引计划
//        	List<BusinessStoreWareDetail> list = businessStoreWareDetailService.getByZidAndType(planBid, "plan");
//        	List<BusinessStoreWareDetail> noticeList = businessStoreWareDetailService.getByZidAndType(id, "notice");
//        	for (BusinessStoreWareDetail businessStoreWareDetail : list) {
//        		for (BusinessStoreWareDetail noticeDetail : noticeList) {
//        			if(noticeDetail.getGrainAnnual().equals(businessStoreWareDetail.getGrainAnnual())  && 
//        					noticeDetail.getGrainAttribute().equals(businessStoreWareDetail.getGrainAttribute())  &&	
//        					noticeDetail.getGrainDetailKind().equals(businessStoreWareDetail.getGrainDetailKind())  &&
//        					noticeDetail.getGrainKind().equals(businessStoreWareDetail.getGrainKind())  &&
//        					noticeDetail.getGrainGrade().equals(businessStoreWareDetail.getGrainGrade())  &&
//        					noticeDetail.getGrainProducingArea().equals(businessStoreWareDetail.getGrainProducingArea())){
//        				
//        				if(noticeDetail.getInCount()!=null){
//        					Integer a = Integer.valueOf(noticeDetail.getInCount())+Integer.valueOf(businessStoreWareDetail.getRemainingNumber());
//        					businessStoreWareDetail.setRemainingNumber(String.valueOf(a));
//        				}
//        				if(noticeDetail.getOutCount()!=null){
//        					Integer b = Integer.valueOf(noticeDetail.getOutCount())+Integer.valueOf(businessStoreWareDetail.getOutRemainingNumber());
//        					businessStoreWareDetail.setOutRemainingNumber(String.valueOf(b));
//        				}
//        				int z = businessStoreWareDetailService.updateById(businessStoreWareDetail);
//        			}
//        			
//				}
//        	}
//        	
//        }else{
//        	//只引合同 或者 引合同有对应的计划
//        	List<BusinessStoreWareDetail> contractList = businessStoreWareDetailService.getByZidAndType(contractBid, "contract");
//        	List<BusinessStoreWareDetail> noticeList = businessStoreWareDetailService.getByZidAndType(id, "notice");
//
//        	for (BusinessStoreWareDetail businessStoreWareDetail : contractList) {
//        		for (BusinessStoreWareDetail noticeDetail : noticeList) {
//        			if(noticeDetail.getGrainAnnual().equals(businessStoreWareDetail.getGrainAnnual())  && 
//        					noticeDetail.getGrainAttribute().equals(businessStoreWareDetail.getGrainAttribute())  &&	
//        					noticeDetail.getGrainDetailKind().equals(businessStoreWareDetail.getGrainDetailKind())  &&
//        					noticeDetail.getGrainKind().equals(businessStoreWareDetail.getGrainKind())  &&
//        					noticeDetail.getGrainGrade().equals(businessStoreWareDetail.getGrainGrade())  &&
//        					noticeDetail.getGrainProducingArea().equals(businessStoreWareDetail.getGrainProducingArea())){
//        				
//        				if(noticeDetail.getInCount()!=null){
//        					Integer a = Integer.valueOf(noticeDetail.getInCount())+Integer.valueOf(businessStoreWareDetail.getRemainingNumber());
//        					businessStoreWareDetail.setRemainingNumber(String.valueOf(a));
//        				}
//        				if(noticeDetail.getOutCount()!=null){
//        					Integer b = Integer.valueOf(noticeDetail.getOutCount())+Integer.valueOf(businessStoreWareDetail.getOutRemainingNumber());
//        					businessStoreWareDetail.setOutRemainingNumber(String.valueOf(b));
//        				}
//        				int z = businessStoreWareDetailService.updateById(businessStoreWareDetail);
//        			}
//        			
//				}
//        	}
//
//        }
    	
    	// 删除主数据.
        businessDeliveryStorageNoticeMapper.deleteByPrimaryKey(id);
        
        // 删除明细列表数据.
        businessStoreWareDetailService.deleteByTypeAndZid(StoreWareType.notice.getValue(), id);
        
        // 删除工作流数据.
        try {
            if (deleteReason!=null) {
                actTaskService.delRunTask(processInstanceId, deleteReason);
            } else {
                deleteReason="删除原因无";
                actTaskService.delRunTask(processInstanceId, deleteReason);
            }
        } catch (Exception e) {
            e.printStackTrace();
        }
    }

    // 提交
    @Transactional(readOnly = false)
    public Map<String, Object> submit(BusinessDeliveryStorageNotice businessDeliveryStorageNotice, String nextUserId, Integer userId,String realName) {
        /*// 当前登录人的id
        Integer userId = userInfo.getUserId();
        // 当前登录人的姓名
        String realName = userInfo.getRealName();*/
        Map<String, Object> returnMap = new HashMap<String, Object>();
        returnMap.put("success", "success");
        // 获取业务数据.
        BusinessDeliveryStorageNotice businessDeliveryStorageNoticeB = businessDeliveryStorageNoticeMapper.selectByPrimaryKey(businessDeliveryStorageNotice.getId());
        
        if (businessDeliveryStorageNoticeB == null) {
            throw new RuntimeException("没有找到相应的数据");
        }
        
        // 获取当前的审批状态，查看是否已经被提交.
        String auditState = businessDeliveryStorageNoticeB.getAuditState();
        if (StringUtils.isNotBlank(auditState) 
                && !auditState.equals(AuditType.UNSUBMIT.getAuditCode()) 
                && !auditState.equals(AuditType.REJECT.getAuditCode())) {
            returnMap.put("success", "error");
            returnMap.put("msg", "您已经提交该数据，无法再次提交！");
            return returnMap;
        }
        
        // 获取到当前进行到的任务节点.
//        Task singleResult = taskService.createTaskQuery().processInstanceId(businessDeliveryStorageNoticeB.getProcessInstanceId()).singleResult();
        String taskNode = actTaskService.taskNode(businessDeliveryStorageNoticeB.getProcessInstanceId());
        JSONObject taskNodeNew = JSONObject.parseObject(taskNode);
        String taskId = taskNodeNew.getString("taskId");
        String taskName = taskNodeNew.getString("taskName");
        
        // 修改流程.
        Map<String, Object> vars = Maps.newHashMap();
        vars.put("pass", "1");
        vars.put("inputUser", nextUserId);
        
        actTaskService.complete(taskId, businessDeliveryStorageNoticeB.getProcessInstanceId(), "", "", vars);

        // 设置审批状态为，审批中.
        businessDeliveryStorageNoticeB.setAuditState(AuditType.SUBMIT.getAuditCode());
        // 修改.
        businessDeliveryStorageNoticeMapper.updateByPrimaryKey(businessDeliveryStorageNoticeB);
        
        // 上一次流程结束，下一流程开始，当前流程.
//        singleResult = taskService.createTaskQuery().processInstanceId(businessDeliveryStorageNoticeB.getProcessInstanceId()).singleResult();
        
        // 流程任务类型.
        TaskType taskType = null;
        StringBuilder nameDeliveryStorageNotice=new StringBuilder();
        BusinessApproval approval = new BusinessApproval();
        
        if ("1".equals(businessDeliveryStorageNoticeB.getBillType())) {
            // 入库
            taskType = TaskType.storageNotice;
            
            nameDeliveryStorageNotice.append(TaskType.storageNotice.getTaskName());
            nameDeliveryStorageNotice.append(" 编号: ");
            nameDeliveryStorageNotice.append(businessDeliveryStorageNoticeB.getBillNumber());
            approval.setTaskTypeName(nameDeliveryStorageNotice.toString());
        } else if ("3".equals(businessDeliveryStorageNoticeB.getBillType())) {
            // 出库
            taskType = TaskType.deliveryNotice;
            nameDeliveryStorageNotice.append(TaskType.deliveryNotice.getTaskName());
            nameDeliveryStorageNotice.append(" 编号: ");
            nameDeliveryStorageNotice.append(businessDeliveryStorageNoticeB.getBillNumber());
            approval.setTaskTypeName(nameDeliveryStorageNotice.toString());
            
        }
        
        // 设置流程中间表.
        //下一环节名称
//        String taskName = singleResult.getName();
        //任务id
//        String taskId = singleResult.getId();
        
        // 设置业务id.
        approval.setProjectId(businessDeliveryStorageNoticeB.getId());
        approval.setProcessInstanceId(businessDeliveryStorageNoticeB.getProcessInstanceId());
        approval.setFromDepartment("经营一部");
        approval.setFromPeople(realName);
        approval.setFromUserName(realName);
        approval.setCreateTime(new Date());
        approval.setTaskType(taskType.getTaskType());
        approval.setProjectName(taskType.getTaskName());
        approval.setResult("待审批");
        approval.setIsHide(0);
        
        //保存申请人的信息,时间.
        approval.setApplyName(realName);
        approval.setApplyNameId(userId.toString());
        approval.setApplyTime(new Date());
        
        UserInfo userInfo2=userInfoService.findById(Integer.valueOf(nextUserId));
        // 审批人
        approval.setOperator(nextUserId);
        // 设置审批人的中文名称.
        approval.setOperatorName(userInfo2.getRealName());
        // 工作流,环节名称.
        approval.setTaskName(taskName);
        // 工作流,taskId.
        approval.setTaskId(taskId);
        
        businessApprovalService.add(approval);
        
        return returnMap;
    }
    
    /**
     * 获取审批列表.
     */
    public List<BusinessDeliveryStorageNoticeAuditVO> getApprovalList(Map<String, Object> paramMap) {
        return businessDeliveryStorageNoticeMapper.getApprovalList(paramMap);
    }


    /**
     * 审批
     * @param deliveryStorageNoticeId 出入库通知单id.
     * @param businessApproval 中间表.
     * @param nextUserId 下一个审批人.
     * @param result 审批结果.
     */
    @Transactional(readOnly = false)
    public void audit(Integer deliveryStorageNoticeId, BusinessApproval businessApproval, String nextUserId, String result ,String name) {
        /*//当前登录人的姓名
        String name=userInfo.getUsername();*/
        // 获取中间表数据.
        BusinessApproval oldBusinessApproval = businessApprovalService.findById(businessApproval.getId());
        // 检测是否并发审核.
        String oldResult = oldBusinessApproval.getResult();
        if ("同意".equals(oldResult) || "退回".equals(oldResult)) {
            throw new RuntimeException("该订单已经审核过，不能再次审核！");
        }
        
        String resultS = "";
        if ("0".equals(result)) {
            resultS = "退回";
        } else if ("1".equals(result)) {
            resultS = "同意";
        } else if ("2".equals(result)) {
            resultS = "驳回";
        } else if ("3".equals(result)) {
            resultS = "拒绝";
        }
        
        // 修改当前中间表.
        oldBusinessApproval.setResult(resultS);
        oldBusinessApproval.setContent(businessApproval.getContent());
        SimpleDateFormat sdf = new SimpleDateFormat("yyyy-MM-dd HH:mm:ss");
        String nowF = sdf.format(new Date());
        oldBusinessApproval.setOperateTime(nowF);
        businessApprovalService.updateByPrimaryKeySelective(oldBusinessApproval);
        
        //得到当前的流程实例id
        String processInstanceId = businessApproval.getProcessInstanceId();
        // Task task = actTaskService.selectTask(processInstanceId);
        //String selectTaskId = actTaskService.selectTaskId(processInstanceId);

        String taskNode = actTaskService.taskNode(businessApproval.getProcessInstanceId());
        JSONObject taskNodeNew = JSONObject.parseObject(taskNode);
        String taskId = taskNodeNew.getString("taskId");
        String taskName = taskNodeNew.getString("taskName");
        // 修改流程.
        Map<String, Object> vars = Maps.newHashMap();
        vars.put("pass", result);
        if (result.equals("1")) {
            // 同意.
            vars.put("inputUser", nextUserId);
        } else if(result.equals("0")) {
            // 退回时，退给上一个人.
            String findHistoryTaskName = actTaskService.findHistoryTaskName(processInstanceId);
            vars.put("inputUser", findHistoryTaskName);
        } else if (result.equals("2")) {
            String findHistoryFirstTaskName = actTaskService.findHistoryFirstTaskName(processInstanceId);
            // 驳回环节(获取提交第一个人).
            vars.put("inputUser", findHistoryFirstTaskName);
        } else if (result.equals("3")) {
            // 拒绝.
        	/*String findHistoryTaskName = actTaskService.findHistoryTaskName(processInstanceId);
            vars.put("inputUser", findHistoryTaskName);*/
        } else {
            // 位置环节
            return; 
        }
        actTaskService.complete(taskId, processInstanceId, businessApproval.getContent(), "", vars);
        
        // 判断流程是否结束，processEnd为null，说明已经没有流程可以执行，说明流程结束.
//        ProcessInstance processEnd = actTaskService.isProcessEnd(processInstanceId);
        boolean processEnd = actTaskService.isProcessEndNew(processInstanceId);
        // 新增流程中间表.
//        Task singleResult = taskService.createTaskQuery().processInstanceId(businessApproval.getProcessInstanceId()).singleResult();
        
        BusinessDeliveryStorageNotice businessDeliveryStorageNoticeB = businessDeliveryStorageNoticeMapper.selectByPrimaryKey(deliveryStorageNoticeId);
        if (processEnd) {
            // 审批结束.
            // 审批通过时间.
            businessDeliveryStorageNoticeB.setAgreeTime(new Date());
            if("3".equals(result)){
                businessDeliveryStorageNoticeB.setAuditState(AuditType.REFUSE.getAuditCode());
            }else if ("1".equals(result)) {
                businessDeliveryStorageNoticeB.setAuditState(AuditType.AGREE.getAuditCode());
            }
            // 修改业务数据的的审批状态为2.
            businessDeliveryStorageNoticeMapper.updateByPrimaryKey(businessDeliveryStorageNoticeB);


            BusinessApproval approval = new BusinessApproval();
            approval.setId(oldBusinessApproval.getId());
            approval.setTaskId(taskId);
            approval.setTaskName(taskName);
            businessApprovalService.updateByPrimaryKeySelective(approval);

        } else {
            // 流程未结束.
            if ("0".equals(result)) {
                //resultS = "退回";
            } else if ("2".equals(result)) {
                //resultS = "驳回";
                businessDeliveryStorageNoticeB.setAuditState(AuditType.REJECT.getAuditCode());
                businessDeliveryStorageNoticeMapper.updateByPrimaryKey(businessDeliveryStorageNoticeB);
            }else if("1".equals(result)){

            	
                // 下一环节名称.
//                String taskName = singleResult.getName();
                // 任务id.
//                String taskId = singleResult.getId();
                
                BusinessApproval approval = new BusinessApproval();
                // 设置业务id.
                approval.setProjectId(deliveryStorageNoticeId);
                // 工作流，流程实例id.
                approval.setProcessInstanceId(businessApproval.getProcessInstanceId());
                approval.setFromDepartment("经营一部");
                approval.setFromPeople(name);
                approval.setFromUserName(name);
                approval.setCreateTime(new Date());
                
                // 申请人的信息,时间.
                approval.setApplyName(oldBusinessApproval.getApplyName());
                approval.setApplyNameId(oldBusinessApproval.getApplyNameId());
                approval.setApplyTime(oldBusinessApproval.getCreateTime());
                StringBuilder nameDeliveryStorageNotice=new StringBuilder();
                // 流程任务类型.
                TaskType taskType = null;
                if ("1".equals(businessDeliveryStorageNoticeB.getBillType())) {
                    // 入库
                    taskType = TaskType.storageNotice;
                    nameDeliveryStorageNotice.append(TaskType.storageNotice.getTaskName());
                    nameDeliveryStorageNotice.append(" 编号: ");
                    nameDeliveryStorageNotice.append(businessDeliveryStorageNoticeB.getBillNumber());
                    approval.setTaskTypeName(nameDeliveryStorageNotice.toString());
                } else if ("3".equals(businessDeliveryStorageNoticeB.getBillType())) {
                    // 出库
                    taskType = TaskType.deliveryNotice;
                    nameDeliveryStorageNotice.append(TaskType.deliveryNotice.getTaskName());
                    nameDeliveryStorageNotice.append(" 编号: ");
                    nameDeliveryStorageNotice.append(businessDeliveryStorageNoticeB.getBillNumber());
                    approval.setTaskTypeName(nameDeliveryStorageNotice.toString());
                }
                // 任务类型
                approval.setTaskType(taskType.getTaskType());
                // 任务名称.
                approval.setProjectName(taskType.getTaskName());
                // 审批状态.
                approval.setResult("待审批");
                // 是否隐藏.
                approval.setIsHide(0);
                // 审批人
                approval.setOperator(nextUserId);
                UserInfo userInfo2=userInfoService.findById(Integer.valueOf(nextUserId));
                // 设置审批人的中文名称.
                approval.setOperatorName(userInfo2.getRealName());
                // 工作流,环节名称.
                approval.setTaskName(taskName);
                // 工作流,taskId.
                approval.setTaskId(taskId);
                
                businessApprovalService.add(approval);
                
            }
            
        }
    }
    
    /**
     * 启用.
     * @param id
     */
    @Override
    @Transactional(readOnly = false)
    public Map<String, String> application(Integer id) {
        Map<String, String> returnMap = new HashMap<String, String>();
        BusinessDeliveryStorageNotice deliveryStorageNoticeY = businessDeliveryStorageNoticeMapper.selectByPrimaryKey(id);
        
        // 先进行判断，对应的计划，合同是否是启用的，只有启用的才可以修改.
        if (deliveryStorageNoticeY != null) {
            // 先判断对应的计划是否已经暂停或者终止.
            Integer planBid = deliveryStorageNoticeY.getPlanBid();
            if (planBid != null) {
                BusinessPlan plan = planService.findById(planBid);
                if (plan != null) {
                    Integer inApplication = plan.getInApplication();
                    // 计划的状态是不是启用.
                    if (inApplication != CrkNoticeStatus.ZHIXINGZHONG.getCrkNoticeCode()) {
                        // 不是启用
                        if (inApplication == CrkNoticeStatus.ZANTING.getCrkNoticeCode()) {
                            returnMap.put("error", "对应的计划已经暂停,无法启用该通知单！");
                        } else if (inApplication == CrkNoticeStatus.ZHONGZHI.getCrkNoticeCode()) {
                            returnMap.put("error", "对应的计划已经终止,无法启用该通知单！");
                        }
                        return returnMap;
                    }
                }
            }
            
            // 判断完计划，再判断相应的合同是否已经暂停或者终止.
            Integer rootContractBid = deliveryStorageNoticeY.getRootContractBid();
            if (rootContractBid != null) {
                // 判断合同，因为考虑到合同变更，所以以root_contract_bid 为查询条件，判断合同中root_contract_bid等于root_contract_bid值的最后一条数据.
                BusinessContractExample example = new BusinessContractExample();
                com.chinaitop.depot.business.model.BusinessContractExample.Criteria createCriteria = example.createCriteria();
                
                createCriteria.andRootContractBidEqualTo(rootContractBid);
                // 审批通过.
                createCriteria.andAuditStateEqualTo(AuditType.AGREE.getAuditCode());
                // 变更状态不是，已变更的合同.
                createCriteria.andChangingNotEqualTo("2");
                // 按照审批通过时间排序.
                example.setOrderByClause(" agree_time desc");
                // 分页，正常来说，只有一条.
                PageHelper.startPage(1, 1);
                List<BusinessContract> contractList = contractService.queryByExample(example);
                if (contractList != null && contractList.size() > 0) {
                    BusinessContract contract = contractList.get(0);
                    Integer inApplication = contract.getInApplication();
                    if (inApplication != CrkNoticeStatus.ZHIXINGZHONG.getCrkNoticeCode()) {
                        // 计划的状态不是启用.
                        if (inApplication == CrkNoticeStatus.ZANTING.getCrkNoticeCode()) {
                            returnMap.put("error", "对应的合同已经暂停,无法启用该通知单！");
                        } else if (inApplication == CrkNoticeStatus.ZHONGZHI.getCrkNoticeCode()) {
                            returnMap.put("error", "对应的合同已经终止,无法启用该通知单！");
                        }
                        return returnMap;
                    }
                }
            }
        }
        
        // 启用出入库通知单.
        BusinessDeliveryStorageNotice deliveryStorageNotice = new BusinessDeliveryStorageNotice();
        
        // 修改的内容.
        deliveryStorageNotice.setInApplication(CrkNoticeStatus.ZHIXINGZHONG.getCrkNoticeCode());
        deliveryStorageNotice.setCreateDepot("");
        // 查询条件.
        BusinessDeliveryStorageNoticeExample example = new BusinessDeliveryStorageNoticeExample();
        Criteria createCriteria = example.createCriteria();
        // id.
        createCriteria.andIdEqualTo(id);
        // 不能修改已经执行完毕的通知单的状态.
        createCriteria.andInApplicationNotEqualTo(CrkNoticeStatus.ZHIXINGWANBI.getCrkNoticeCode());
        
        businessDeliveryStorageNoticeMapper.updateByExampleSelective(deliveryStorageNotice, example);
        
        
        // 修改出入库系统通知单状态.
        Map<String,Object> paramMap = new HashMap<String, Object>();
        paramMap.put("tzdh", id);
        paramMap.put("FLOWSTATUS", CrkNoticeStatus.ZHIXINGZHONG.getCrkNoticeCode());
        // 修改出入库系统的 通知单状态.
        businessSceduleService.updateNoticeStatus(paramMap);
        return returnMap;
    }

    /**
     * 暂停.
     * @param id
     */
    @Override
    @Transactional(readOnly = false)
    public void discontinue(Integer id,String refuseReason) {
        // 暂停出入库通知单.
        BusinessDeliveryStorageNotice deliveryStorageNotice = new BusinessDeliveryStorageNotice();
        
        // 修改的内容.
        deliveryStorageNotice.setId(id);
        deliveryStorageNotice.setInApplication(CrkNoticeStatus.ZANTING.getCrkNoticeCode());
        deliveryStorageNotice.setCreateDepot(refuseReason);
        // 查询条件.
        BusinessDeliveryStorageNoticeExample example = new BusinessDeliveryStorageNoticeExample();
        Criteria createCriteria = example.createCriteria();
        // id.
        createCriteria.andIdEqualTo(id);
        // 不能修改已经执行完毕的通知单的状态.
        createCriteria.andInApplicationNotEqualTo(CrkNoticeStatus.ZHIXINGWANBI.getCrkNoticeCode());
        
        businessDeliveryStorageNoticeMapper.updateByExampleSelective(deliveryStorageNotice, example);
        
        // 修改出入库系统通知单状态.
        Map<String,Object> paramMap = new HashMap<String, Object>();
        paramMap.put("tzdh", id);
        paramMap.put("FLOWSTATUS", CrkNoticeStatus.ZANTING.getCrkNoticeCode());
        // 修改出入库系统的 通知单状态.
        businessSceduleService.updateNoticeStatus(paramMap);
    }
    
    /**
     * 终止.
     * @param id
     */
    @Override
    @Transactional(readOnly = false)
    public void finish(Integer id,String refuseReason) {
        // 终止出入库通知单.
        BusinessDeliveryStorageNotice deliveryStorageNotice = new BusinessDeliveryStorageNotice();
        
        // 修改的内容.
        deliveryStorageNotice.setId(id);
        deliveryStorageNotice.setInApplication(CrkNoticeStatus.ZHONGZHI.getCrkNoticeCode());
        deliveryStorageNotice.setCreateDepot(refuseReason);
        // 查询条件.
        BusinessDeliveryStorageNoticeExample example = new BusinessDeliveryStorageNoticeExample();
        Criteria createCriteria = example.createCriteria();
        // id.
        createCriteria.andIdEqualTo(id);
        // 不能修改已经执行完毕的通知单的状态.
        createCriteria.andInApplicationNotEqualTo(CrkNoticeStatus.ZHIXINGWANBI.getCrkNoticeCode());
        
        businessDeliveryStorageNoticeMapper.updateByExampleSelective(deliveryStorageNotice, example);
        
        
        // 修改出入库系统通知单状态.
        Map<String,Object> paramMap = new HashMap<String, Object>();
        paramMap.put("tzdh", id);
        paramMap.put("FLOWSTATUS", CrkNoticeStatus.ZHONGZHI.getCrkNoticeCode());
        // 修改出入库系统的 通知单状态.
        businessSceduleService.updateNoticeStatus(paramMap);
    }

    /**
     * 查看通知单执行进度.
     */
    @Override
    public PageInfo<BusinessDeliveryStorageNoticeAuditVO> getScheduleList(
            Integer pageNum, Integer pageSize,
            BusinessDeliveryStorageNotice deliveryStorageNotice,String agreeTimeStart,String agreeTimeEnd) {
        // 可以查询所有人通过的通知单
        BusinessDeliveryStorageNoticeExample example = new BusinessDeliveryStorageNoticeExample();
        Criteria createCriteria = example.createCriteria();
        createCriteria.andAuditStateEqualTo(AuditType.AGREE.getAuditCode());
        List<String> list = new ArrayList<String>();
        list.add("1");
        list.add("3");
        createCriteria.andBillTypeIn(list);
        if (deliveryStorageNotice != null) {
            // 凭证类型.
            if (StringUtils.isNotBlank(deliveryStorageNotice.getBillType())) {
                createCriteria.andBillTypeEqualTo(deliveryStorageNotice.getBillType());
            }
            
            // 计划编号.
            if (StringUtils.isNotBlank(deliveryStorageNotice.getPlanNumber())) {
                createCriteria.andPlanNumberLike("%" + deliveryStorageNotice.getPlanNumber() + "%");
            }
            // 合同编号.
            if (StringUtils.isNotBlank(deliveryStorageNotice.getContract())) {
                createCriteria.andContractLike("%" + deliveryStorageNotice.getContract() + "%");
            }
            // 通知单编号.
            if (StringUtils.isNotBlank(deliveryStorageNotice.getBillNumber())) {
                createCriteria.andBillNumberLike("%" + deliveryStorageNotice.getBillNumber() + "%");
            }
            
            // 计划id.
            if (deliveryStorageNotice.getPlanBid() != null) {
                createCriteria.andPlanBidEqualTo(deliveryStorageNotice.getPlanBid());
            }
            
            // 原始合同id.
            if (deliveryStorageNotice.getRootContractBid() != null) {
                createCriteria.andRootContractBidEqualTo(deliveryStorageNotice.getRootContractBid());
            }
        }
        
        SimpleDateFormat simpleDateFormat = new SimpleDateFormat("yyyy-MM-dd HH:mm:ss");
        try {
        	if(agreeTimeStart!=null && agreeTimeStart!=""){
        		agreeTimeStart = agreeTimeStart+" "+"00:00:00";
        		Date startDate = simpleDateFormat.parse(agreeTimeStart);
        		createCriteria.andAgreeTimeGreaterThanOrEqualTo(startDate);
        	}
        	if(agreeTimeEnd!=null && agreeTimeEnd!=""){
        		agreeTimeEnd = agreeTimeEnd +" "+ "23:59:59";
        		Date endtDate = simpleDateFormat.parse(agreeTimeEnd);
        		createCriteria.andAgreeTimeLessThanOrEqualTo(endtDate);
        	}
        	
		} catch (ParseException e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
		}
        
        // 审批通过时间倒序.
        example.setOrderByClause("agree_time desc");
        PageInfo<BusinessDeliveryStorageNoticeAuditVO> pageInfo = new PageInfo<BusinessDeliveryStorageNoticeAuditVO>();
        // 分页.
        if (pageNum != null && pageSize != null) {
             PageHelper.startPage(pageNum, pageSize);
         }
        
        List<BusinessDeliveryStorageNoticeAuditVO> noticeList = businessDeliveryStorageNoticeMapper.selectByExampleToVO(example);
        //提货单
        /*if(!((deliveryStorageNotice.getRootContractBid() != null || deliveryStorageNotice.getPlanBid() != null) && noticeList.size()>0)){
        	//合同里面的进度要查看通知单  如果有RootContractBid  并且noticeList 有值 则说明是通知单表的 则不执行查询提货单的
        	BusinessLadingExample exampleLading = new BusinessLadingExample();
        	List<BusinessLading> list = ladingService.queryByExample(exampleLading); 
        	if(list.size()>0 && list!=null){
        		for (int i = 0; i < list.size(); i++) {
        			BusinessDeliveryStorageNoticeAuditVO businessAuditVO = new BusinessDeliveryStorageNoticeAuditVO();
        			businessAuditVO.setId(list.get(i).getId());
        			businessAuditVO.setBillNumber(list.get(i).getLadingNumber());
        			businessAuditVO.setAgreeTime(list.get(i).getBillDate());
        			BigDecimal js = new BigDecimal(list.get(i).getUnloadNumber().toString());
        			BigDecimal zl = new BigDecimal(list.get(i).getSalesDepartment());
        			businessAuditVO.setShipingCount(js.multiply(zl).toString());//要求数量
        			noticeList.add(businessAuditVO);
        		}
        	}
        }*/
        
        if (noticeList != null && noticeList.size() > 0) {
            // 拼接通知单id.
            StringBuffer sb = new StringBuffer();
           // StringBuffer thdNumber = new StringBuffer();
            sb.append("(");
            //thdNumber.append("(");
            for (BusinessDeliveryStorageNoticeAuditVO businessDeliveryStorageNoticeV : noticeList) {
            	/*if(businessDeliveryStorageNoticeV.getBillType()!=null){//通知单
                    sb.append("\"" + businessDeliveryStorageNoticeV.getId() + "\"").append(",");
            	}else{//提货单
            		thdNumber.append("\"" + businessDeliveryStorageNoticeV.getBillNumber() + "\"").append(",");
            	}*/
            	sb.append("\"" + businessDeliveryStorageNoticeV.getId() + "\"").append(",");
            }
            
           String substring = sb.substring(0, sb.length() - 1);
           sb = new StringBuffer(substring);
           List<BusinessScedule> businessSceduleList =null;
            if(sb.length()!=0){
                sb.append(")");
                 businessSceduleList = businessSceduleService.getNoticeSceduleInNoticeId(sb.toString());
                   
             }
            /*String thdNumberString = thdNumber.substring(0, thdNumber.length() - 1);
            thdNumber = new StringBuffer(thdNumberString);
            if(thdNumber.length()!=0){
            	thdNumber.append(")");
            	List<BusinessScedule> thdSceduleList = businessSceduleService.getThdInNumber(thdNumber.toString());
            	 for (BusinessScedule thd : thdSceduleList) {
             		if(thd!=null){
             			businessSceduleList.add(thd);
             		}
     			}
            }	*/
            
           
            
            // 通过通知单id,去出入库查询.
            if (businessSceduleList != null && businessSceduleList.size() > 0) {
                // 循环通知单list.
                for (BusinessDeliveryStorageNoticeAuditVO businessDeliveryStorageNoticeV : noticeList) {
                    // 循环出入库，查询的list.
                    Boolean flag = false;
                    for (BusinessScedule businessScedule : businessSceduleList) {
	                    if(businessScedule!=null){
	                    		
	                    	if(businessScedule.getCrkLx()==null){//提货单
	                    		if(businessDeliveryStorageNoticeV.getBillNumber().equals(businessScedule.getThdh())){
	                    			 flag = true;
	                    			// 在出入库系统中，获取完成数量
	                    			 String sumThdsl = businessScedule.getSumWcsl();
	                                 BigDecimal sumThdslD = new BigDecimal(sumThdsl);
	                                 // 赋值：设置完成数量. stripTrailingZeros() 去除结尾的0
	                                 businessDeliveryStorageNoticeV.setCompleteAmount(sumThdslD.stripTrailingZeros().toPlainString());
	                                 // 通知单要求数量.
	                                 String demandAmount = businessDeliveryStorageNoticeV.getShipingCount();
	                                 BigDecimal demandAmountD = new BigDecimal(demandAmount);
	                                 
	                                 // 除法. 已完成数量/要求数量
	                                 BigDecimal complete = sumThdslD.divide(demandAmountD, 4, BigDecimal.ROUND_HALF_DOWN);
	                                 // 乘以100，得到的是百分比.
	                                 BigDecimal completeRate = complete.multiply(new BigDecimal("100"));
	                                 // 赋值：设置完成百分比.
	                                 businessDeliveryStorageNoticeV.setCompleteRate(completeRate.stripTrailingZeros().toPlainString());
	                    		}
		                    		// 在出入库list中删除该数据，这样下次循环数量就会减少了.
		                			businessSceduleList.remove(businessScedule);
		                			// 跳出当前的出入库循环.
		                			break;
	                    	}else{//通知单
	                    		if (businessDeliveryStorageNoticeV.getId().toString().equals(businessScedule.getTzdId())) {
	                    			flag = true;
	                    			// 在出入库系统中，存在该通知单对应的出入库数据，进行计算，得出完成比率.
	                    			String crkLx = businessScedule.getCrkLx();
	                    			if ("1".equals(crkLx)) {
	                    				// 入库
	                    				// 入库数量.
	                    				String sumRksl = businessScedule.getSumWcsl();
	                    				BigDecimal sumRkslD = new BigDecimal(sumRksl);
	                    				// 赋值：设置完成数量. stripTrailingZeros() 去除结尾的0
	                    				businessDeliveryStorageNoticeV.setCompleteAmount(sumRkslD.stripTrailingZeros().toPlainString());
	                    				// 通知单要求数量.
	                    				String demandAmount = businessDeliveryStorageNoticeV.getShipingCount();
	                    				BigDecimal demandAmountD = new BigDecimal(demandAmount);
	                    				
	                    				// 除法. 已完成数量/要求数量
	                    				BigDecimal complete = sumRkslD.divide(demandAmountD, 4, BigDecimal.ROUND_HALF_DOWN);
	                    				// 乘以100，得到的是百分比.
	                    				BigDecimal completeRate = complete.multiply(new BigDecimal("100"));
	                    				// 赋值：设置完成百分比.
	                    				businessDeliveryStorageNoticeV.setCompleteRate(completeRate.stripTrailingZeros().toPlainString());
	                    			} else if ("3".equals(crkLx)) {
	                    				// 出库
	                    				// 出库数量.
	                    				String sumCksl = businessScedule.getSumWcsl();
	                    				BigDecimal sumCkslD = new BigDecimal(sumCksl);
	                    				
	                    				// 赋值：设置完成数量. stripTrailingZeros() 去除结尾的0
	                    				businessDeliveryStorageNoticeV.setCompleteAmount(sumCkslD.stripTrailingZeros().toPlainString());
	                    				
	                    				// 通知单要求数量.
	                    				String demandAmount = businessDeliveryStorageNoticeV.getShipingCount();
	                    				BigDecimal demandAmountD = new BigDecimal(demandAmount);
	                    				
	                    				// 除法. 已完成数量/要求数量
	                    				BigDecimal complete = sumCkslD.divide(demandAmountD, 4, BigDecimal.ROUND_HALF_DOWN);
	                    				// 乘以100，得到的是百分比.
	                    				BigDecimal completeRate = complete.multiply(new BigDecimal("100"));
	                    				// 赋值：设置完成百分比.
	                    				businessDeliveryStorageNoticeV.setCompleteRate(completeRate.stripTrailingZeros().toPlainString());
	                    			}
	                    			// 在出入库list中删除该数据，这样下次循环数量就会减少了.
	                    			businessSceduleList.remove(businessScedule);
	                    			// 跳出当前的出入库循环.
	                    			break;
	                    		}
	                    	}
	                    }
                   }
                    if (!flag) {
                        // 进行到这里，说明出入库系统中，不存在该通知单对应的出入库数据. 设置完成数量，和完成比例为0.
                        businessDeliveryStorageNoticeV.setCompleteAmount("0");
                        businessDeliveryStorageNoticeV.setCompleteRate("0");
                    }
                }
            } else {
                // 没有找到任何出入库数据. 设置完成数量，和完成比例为0.
                for (BusinessDeliveryStorageNoticeAuditVO businessDeliveryStorageNoticeV : noticeList) {
                    businessDeliveryStorageNoticeV.setCompleteAmount("0");
                    businessDeliveryStorageNoticeV.setCompleteRate("0");
                }
            }
            double combinedComplete = 0,combinedRequired = 0;
            for (int i = 0; i < noticeList.size(); i++) {
                combinedComplete += Double.parseDouble(noticeList.get(i).getCompleteAmount());//完成数量(公斤)
                combinedRequired += Double.parseDouble(noticeList.get(i).getShipingCount());//要求数量(公斤)
                if(i == 9){
                    break;
                }
            }
            if(noticeList.size() > 0){
            	noticeList.get(0).setCombinedComplete(combinedComplete);
            	noticeList.get(0).setCombinedRequired(combinedRequired);
            }
            pageInfo = new PageInfo<BusinessDeliveryStorageNoticeAuditVO>(noticeList);
        }
        return pageInfo;
    }


    @Override
    public List<BusinessDeliveryStorageNotice> queryByExample(Integer pageNum,
            Integer pageSize, Integer userId,
            BusinessDeliveryStorageNotice notice, String order) {
        BusinessDeliveryStorageNoticeExample example = new BusinessDeliveryStorageNoticeExample();
        Criteria createCriteria = example.createCriteria();
        Criteria createCriteria1 = example.createCriteria();
        // 用户id.
        if (userId != null) {
            createCriteria.andCreaterIdEqualTo(userId);
            createCriteria1.andCreaterIdEqualTo(userId);
        }
        if (notice != null) {
            // 凭证类型.
            if (StringUtils.isNotBlank(notice.getBillType())) {
                createCriteria.andBillTypeEqualTo(notice.getBillType());
                createCriteria1.andBillTypeEqualTo(notice.getBillType());
            }
            
            // 通知单编号.
            if (StringUtils.isNotBlank(notice.getBillNumber())) {
                createCriteria.andBillNumberLike("%" + notice.getBillNumber() + "%");
                createCriteria1.andBillNumberLike("%" + notice.getBillNumber() + "%");
            }
            
            // 计划id.
            if (notice.getPlanBid() != null) {
                createCriteria.andPlanBidEqualTo(notice.getPlanBid());
                createCriteria1.andPlanBidEqualTo(notice.getPlanBid());
            }
            
            // 客户名称.
            if (StringUtils.isNotBlank(notice.getDeliveryCustomer())) {
                createCriteria.andDeliveryCustomerLike("%" + notice.getDeliveryCustomer() + "%");
                createCriteria1.andDeliveryCustomerLike("%" + notice.getDeliveryCustomer() + "%");
            }
            
            // 计划审批状态.
            if (StringUtils.isNotBlank(notice.getAuditState())){
                createCriteria.andAuditStateEqualTo(notice.getAuditState());
                createCriteria1.andAuditStateEqualTo(notice.getAuditState());
            }
            
            //主库还是代储库的通知单
            if(notice.getOrgId()!=null){
            		createCriteria.andOrgIdEqualTo(notice.getOrgId());
            		createCriteria1.andHouseIdEqualTo(notice.getOrgId());
            }
        }
        createCriteria.andBillTypeNotEqualTo("2");
        createCriteria1.andBillTypeNotEqualTo("2");
        example.or(createCriteria1);
        if (StringUtils.isNotBlank(order)) {
            example.setOrderByClause(order);
        }
        PageHelper.startPage(pageNum, pageSize);
        List<BusinessDeliveryStorageNotice> list = businessDeliveryStorageNoticeMapper.selectByExample(example);
        return list;
    }
    
    //中转
    @Override
    public List<BusinessDeliveryStorageNotice> queryByExampleTransfer(Integer pageNum,
    		Integer pageSize, Integer userId,
    		BusinessDeliveryStorageNotice notice, String order) {
    	BusinessDeliveryStorageNoticeExample example = new BusinessDeliveryStorageNoticeExample();
    	Criteria createCriteria = example.createCriteria();
    	// 用户id.
    	if (userId != null) {
    		createCriteria.andCreaterIdEqualTo(userId);
    	}
    	if (notice != null) {
    		// 凭证类型.
    		if (StringUtils.isNotBlank(notice.getBillType())) {
    			createCriteria.andBillTypeEqualTo(notice.getBillType());
    		}
    		
    		// 通知单编号.
    		if (StringUtils.isNotBlank(notice.getBillNumber())) {
    			createCriteria.andBillNumberLike("%" + notice.getBillNumber() + "%");
    		}
    		
    		// 计划id.
    		if (notice.getPlanBid() != null) {
    			createCriteria.andPlanBidEqualTo(notice.getPlanBid());
    		}
    		
    		// 客户名称.
    		if (StringUtils.isNotBlank(notice.getDeliveryCustomer())) {
    			createCriteria.andDeliveryCustomerLike("%" + notice.getDeliveryCustomer() + "%");
    		}
    		
    		// 计划审批状态.
    		if (StringUtils.isNotBlank(notice.getAuditState())){
    			createCriteria.andAuditStateEqualTo(notice.getAuditState());
    		}
    	}
    	createCriteria.andBillTypeEqualTo("2");
    	createCriteria.andOrgIdEqualTo(notice.getOrgId());
    	if (StringUtils.isNotBlank(order)) {
    		example.setOrderByClause(order);
    	}
    	PageHelper.startPage(pageNum, pageSize);
    	List<BusinessDeliveryStorageNotice> list = businessDeliveryStorageNoticeMapper.selectByExample(example);
    	return list;
    }

    @Override
    public void updateApplication(Map<String, Object> paramMap) {
        businessDeliveryStorageNoticeMapper.updateApplication(paramMap);
    }

    
    /**
     * 入站通知单新增
     */
    @Transactional(readOnly = false)
    public Map<String, Object> addTransfer(BusinessDeliveryStorageNotice businessDeliveryStorageNotice, 
            List<BusinessStoreWareDetail> storeWareDetailList, Integer userId, String name ,Integer orgId) {
        Map<String, Object> returnMap = new HashMap<String, Object>();
        /*//当前登录人的id
        Integer userId = null;
        //当前登录人的姓名
        String name = null;
        try {
            userId = userInfo.getUserId();
            name = userInfo.getRealName();
        } catch (Exception e) {
            e.printStackTrace();
            returnMap.put("error", "获取不到登录人信息，请重新登录！");
            return returnMap;
        }*/
        
        // 设置登录人信息.
        businessDeliveryStorageNotice.setBillDate(new Date());
        businessDeliveryStorageNotice.setCreateTime(new Date());
        businessDeliveryStorageNotice.setBillMan(name);
        businessDeliveryStorageNotice.setCreater(name);
        businessDeliveryStorageNotice.setCreaterId(userId);
        businessDeliveryStorageNotice.setOrgId(orgId);
        //默认审批已通过(不用审批)
        businessDeliveryStorageNotice.setInApplication(1);
        
        
       
        // 保存.
        businessDeliveryStorageNoticeMapper.insert(businessDeliveryStorageNotice);
        
         
        //businessDeliveryStorageNoticeMapper.updateByPrimaryKey(businessDeliveryStorageNotice);
        
        // 新增仓房货位数量质量标准子表.
        if (storeWareDetailList != null && storeWareDetailList.size() > 0) {
            for (BusinessStoreWareDetail storeWareDetail : storeWareDetailList) {
                storeWareDetail.setZid(businessDeliveryStorageNotice.getId());
                storeWareDetail.setType(StoreWareType.transferNotice.getValue());
                storeWareDetail.setCreateTime(new Date());
                businessStoreWareDetailService.add(storeWareDetail);
            }
        }
        
        return returnMap;
    }


	@Override
	public boolean isQuoteContract(Integer contract_id) {
		boolean flag = false;
		BusinessDeliveryStorageNoticeExample example = new BusinessDeliveryStorageNoticeExample();
		Criteria criteria = example.createCriteria();
		criteria.andContractBidEqualTo(contract_id);
		List<BusinessDeliveryStorageNotice> list = businessDeliveryStorageNoticeMapper.selectByExample(example);
		if (null != list && list.size() > 0) {
			flag = true;
		}
		return flag;
	}

    
}
