package com.chinaitop.depot.pushs.service.impl;

import java.lang.reflect.Parameter;
import java.util.*;

import javax.annotation.Resource;

import com.chinaitop.depot.business.mapper.BusinessStoreWareDetailMapper;
import com.chinaitop.depot.business.model.*;
import com.chinaitop.depot.utils.ParameterUtil;
import org.apache.commons.lang.ArrayUtils;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;

import com.alibaba.fastjson.JSON;
import com.chinaitop.depot.business.mapper.BusinessContractMapper;
import com.chinaitop.depot.business.mapper.BusinessDeliveryStorageNoticeMapper;
import com.chinaitop.depot.business.mapper.BusinessLadingMapper;
import com.chinaitop.depot.pushs.service.PushsService;
import com.chinaitop.depot.unissoft.model.ResponseEntity;

@Service
public class PushServiceImpl implements PushsService{
	
    final static Logger logger  =  LoggerFactory.getLogger(PushServiceImpl.class );


	@Resource
    private BusinessContractMapper businessContractMapper;
	@Resource
	private BusinessDeliveryStorageNoticeMapper businessDeliveryStorageNoticeMapper;
	@Resource
	private BusinessStoreWareDetailMapper businessStoreWareDetailMapper;
	@Resource
    private BusinessLadingMapper ladingMapper;



	@Transactional(rollbackFor = Exception.class)
	@Override
	public String updateContractStatus(String inApplication, String ids) {
		Map<String,Object> mapData = new HashMap<String, Object>();
		//添加查询合同id方法
		BusinessContractExample businessContractExample = new BusinessContractExample();
		BusinessContractExample.Criteria criteria = businessContractExample.createCriteria();
		//传输过来的ids转换成list<Integer>类型的。
		int[] intIds = Arrays.stream(ids.split(",")).mapToInt(Integer::parseInt).toArray();
		Integer[] IntegerIds = ArrayUtils.toObject(intIds);
		criteria.andIdIn(Arrays.asList(IntegerIds));
//		businessContractExample.setOrderByClause("create_time DESC");
		List<BusinessContract> businessContracts = businessContractMapper.selectByExample(businessContractExample);
		if(businessContracts.size() == 0){
			mapData.put("error", "id is error");
			return JSON.toJSONString(ResponseEntity.ok(mapData));
		}
//		criteria.andCreateTimeEqualTo(businessContracts.get(0).getChangeTime());
		//添加要修改的状态值
		BusinessContract businessContract = new BusinessContract();
		businessContract.setInApplication(Integer.parseInt(inApplication));
		businessContractMapper.updateByExampleSelective(businessContract,businessContractExample);

		mapData.put("status", "success");
		return JSON.toJSONString(ResponseEntity.ok(mapData));
	}



	@Transactional(rollbackFor = Exception.class)
	@Override
	public String getNoticeDatas(String orgId, String inApplication, String ids ) {
		List<String> listType=new ArrayList<String>();
		listType.add("notice");
		listType.add("transferNotice");
		//先查出主数据
		Map<String,Object> mapCondition = new HashMap<String, Object>();
		mapCondition.put("orgId",orgId);
		mapCondition.put("inApplication",inApplication);
		if(ParameterUtil.isnotnull(ids)) {
			List<String> idsList = Arrays.asList(ids.split(","));
			mapCondition.put("ids", idsList);
		}
		List<Map<String,Object>> mainList =businessDeliveryStorageNoticeMapper.uploadNoticeData(mapCondition);
		//再根据主数据id查出相关的从数据插入
		for (Map<String,Object> mains : mainList) {
			BusinessStoreWareDetailExample businessStoreWareDetailExample = new BusinessStoreWareDetailExample();
			BusinessStoreWareDetailExample.Criteria criteria = businessStoreWareDetailExample.createCriteria();
			criteria.andZidEqualTo(Integer.valueOf(mains.get("id").toString()));
			criteria.andTypeIn(listType);//默认查询通知单类型数据
			List<BusinessStoreWareDetail> json = businessStoreWareDetailMapper.selectByExample(businessStoreWareDetailExample);
			mains.put("fromData",json);//从表数据以json形式插入主表数据
		}
		
		Map<String,Object> mapCondition1 = new HashMap<String, Object>();
		mapCondition1.put("orgId",orgId);
		if(ParameterUtil.isnotnull(ids)) {
			List<String> idsList = Arrays.asList(ids.split(","));
			mapCondition1.put("ids", idsList);
		}
		List<Map<String,Object>> mainList1 =businessDeliveryStorageNoticeMapper.getNoticeData(mapCondition1);

        Map<String,Object> mapData = new HashMap<String, Object>();
		mapData.put("status", "success");
		mapData.put("mainData", mainList);
		mapData.put("mainData1", mainList1);
		return JSON.toJSONString(ResponseEntity.ok(mapData));
	}

	@Transactional(rollbackFor = Exception.class)
	@Override
	public String updateNoticeState(String inApplication, String ids) {
		//添加查询通知单id方法
		BusinessDeliveryStorageNoticeExample noticeExample = new BusinessDeliveryStorageNoticeExample();
		com.chinaitop.depot.business.model.BusinessDeliveryStorageNoticeExample.Criteria noticeCriteria = noticeExample.createCriteria();
		//传输过来的ids转换成list<Integer>类型的。
		int[] intIds = Arrays.stream(ids.split(",")).mapToInt(Integer::parseInt).toArray();
		Integer[] IntegerIds = ArrayUtils.toObject(intIds);
		noticeCriteria.andIdIn(Arrays.asList(IntegerIds));
		//添加要修改的状态值
		BusinessDeliveryStorageNotice businessDeliveryStorageNotice = new BusinessDeliveryStorageNotice();
		businessDeliveryStorageNotice.setInApplication(Integer.parseInt(inApplication));
		businessDeliveryStorageNoticeMapper.updateByExampleSelective(businessDeliveryStorageNotice,noticeExample);

		Map<String,Object> mapData = new HashMap<String, Object>();
		mapData.put("status", "success");
		return JSON.toJSONString(ResponseEntity.ok(mapData));
	}

	@Transactional(rollbackFor = Exception.class)
	@Override
	public String getContractDatas(String orgId, String inApplication, String ids) {
		//查出数据
		Map<String,Object> mapCondition = new HashMap<String, Object>();
		mapCondition.put("inApplication",inApplication);
		if(ParameterUtil.isnotnull(ids)){
			List<String> conList = Arrays.asList(ids.split(","));
			mapCondition.put("ids",conList);
		}
		if(ParameterUtil.isnotnull(orgId)){
			mapCondition.put("orgId",orgId);
		}
		mapCondition.put("contractType",6913);//只查询中转合同
		mapCondition.put("auditState",2);//审批通过的中转合同
		List<Map<String,Object>> mainList =businessContractMapper.contractData(mapCondition);

		//再根据主数据id查出相关的从数据插入
		for (Map<String,Object> mains : mainList) {
			BusinessStoreWareDetailExample businessStoreWareDetailExample = new BusinessStoreWareDetailExample();
			BusinessStoreWareDetailExample.Criteria criteria = businessStoreWareDetailExample.createCriteria();
			criteria.andZidEqualTo(Integer.valueOf(mains.get("id").toString()));
			criteria.andTypeEqualTo("contract");//默认查询通知单类型数据
			criteria.andOrgIdEqualTo(Integer.parseInt(orgId));//查询组织机构id
			List<BusinessStoreWareDetail> json = businessStoreWareDetailMapper.selectByExample(businessStoreWareDetailExample);
			mains.put("fromData",json);//从表数据以json形式插入主表数据
		}

		Map<String,Object> mapData = new HashMap<String, Object>();
		mapData.put("status", "success");
		mapData.put("mainData", mainList);
		return JSON.toJSONString(ResponseEntity.ok(mapData));
	}

	@Transactional(rollbackFor = Exception.class)
	@Override
	public String isHaveTzd(String orgId, String inApplication) {
		//先查出数据
		BusinessDeliveryStorageNoticeExample businessDeliveryStorageNoticeExample = new BusinessDeliveryStorageNoticeExample();
		BusinessDeliveryStorageNoticeExample.Criteria criteria = businessDeliveryStorageNoticeExample.createCriteria();
		//criteria.andOrgIdEqualTo(Integer.parseInt(orgId));
		criteria.andHouseIdEqualTo(Integer.parseInt(orgId)); //若是主库orgId和houseId是一样的  否则houseId是代储库的   orgId是主库
		criteria.andInApplicationEqualTo(Integer.parseInt(inApplication));
		List<BusinessDeliveryStorageNotice> mainList =businessDeliveryStorageNoticeMapper.selectByExample(businessDeliveryStorageNoticeExample);
		//把查出来的所有数据定义一个数组
		List<String> ids = new ArrayList<>();
		for (int i = 0; i < mainList.size(); i++) {
			ids.add(mainList.get(i).getId().toString());
		}
		Map<String,Object> mapData = new HashMap<String, Object>();
		mapData.put("status", "success");
		mapData.put("ids", ids);
		return JSON.toJSONString(ResponseEntity.ok(mapData));
	}



	@Override
	public String getLadingDatas(String orgId, String thdhs) {
		
		Map<String,Object> mapCondition = new HashMap<String, Object>();
		mapCondition.put("orgId",orgId);
		if(ParameterUtil.isnotnull(thdhs)) {
			List<String> thdhsList = Arrays.asList(thdhs.split(","));
			mapCondition.put("thdhs", thdhsList);
		}
		
		List<Map<String,Object>> mainList =ladingMapper.getLadingDatas(mapCondition);

        Map<String,Object> mapData = new HashMap<String, Object>();
		mapData.put("status", "success");
		mapData.put("mainData", mainList);
		return JSON.toJSONString(ResponseEntity.ok(mapData));
	}
}
